@@ -11,8 +11,6 @@
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
-#
-# ================================================================================================
 
 import collections.abc as abc
 import itertools
@@ -22,6 +20,7 @@ import numpy as np
 import dimod
 
 from collections import defaultdict
+from copy import deepcopy
 
 from dwave.embedding.chain_breaks import majority_vote, broken_chains
 from dwave.embedding.exceptions import MissingEdgeError, MissingChainError, InvalidNodeError, DisconnectedChainError
@@ -29,6 +28,16 @@ from dwave.embedding.utils import adjacency_to_edges, intlabel_disjointsets
 from dwave.embedding.chain_strength import uniform_torque_compensation
 
 
+try:
+    from dimod import AdjArrayBQM
+    from dimod.core.bqm import BinaryView, SpinView
+except ImportError:
+    # dummy classes for isinstance checks
+    class AdjArrayBQM: pass
+    class BinaryView: pass
+    class SpinView: pass
+
+
 __all__ = ['embed_bqm',
            'embed_ising',
            'embed_qubo',
@@ -58,7 +67,7 @@ class EmbeddedStructure(dict):
 
     def __init__(self, target_edges, embedding):
         if isinstance(embedding, EmbeddedStructure):
-            super().__init__(self)
+            super().__init__(embedding)
             if target_edges is None:
                 # this condition is used by self.copy
                 self._interaction_edges = embedding._interaction_edges.copy()
@@ -98,6 +107,24 @@ class EmbeddedStructure(dict):
             if len(emb_u) != disjoint_sets[u].size(0):
                 raise DisconnectedChainError(u)
 
+    def __copy__(self):
+        return EmbeddedStructure(None, self)
+
+    copy = __copy__
+
+    def __deepcopy__(self, memo):
+        id_ = id(self)
+        new = memo.get(id_, None)
+        if new is not None:
+            return new
+
+        # everything is hashable so all copies are deep. Technically this
+        # can break some object equivalence (because we're ignoring the
+        # memo) but since the chains are converted into a new tuple in
+        # the __init__ already... this seems simpler and cleaner.
+        new = memo[id_] = self.copy()
+        return new
+
     @property
     def chain_strength(self):
         """callable: Return the chain strength selected for embedding."""
@@ -153,9 +180,6 @@ class EmbeddedStructure(dict):
 
     __delitem__=__setitem__=clear=pop=popitem=setdefault=update=_mutate_dict
 
-    def copy(self):
-        return EmbeddedStructure(None, self)
-
     def fromkeys(self, *args, **kwargs):
         """Raise a NotImplemented -- this method is not supported for the
         EmbeddedStructure class, but exists because dict is the parent class."""
@@ -170,13 +194,11 @@ class EmbeddedStructure(dict):
                 Binary quadratic model to embed.
 
             chain_strength (float/mapping/callable, optional):
-                Magnitude of the quadratic bias (in SPIN-space) applied between
-                variables to create chains, with the energy penalty of chain
-                breaks set to 2 * `chain_strength`.  If a mapping is passed, a
-                chain-specific strength is applied.  If a callable is passed, it
-                will be called on `chain_strength(source_bqm, self)`
-                and should return a float or mapping, to be interpreted as above.
-                By default, `chain_strength` is calculated with 
+                Sets the coupling strength between qubits representing variables 
+                that form a :term:`chain`. Mappings should specify the required 
+                chain strength for each variable. Callables should accept the BQM 
+                and embedding and return a float or mapping. By default, 
+                `chain_strength` is calculated with
                 :func:`~dwave.embedding.chain_strength.uniform_torque_compensation`.
 
             smear_vartype (:class:`.Vartype`, optional, default=None):
@@ -222,15 +244,21 @@ class EmbeddedStructure(dict):
         else:
             smear_vartype = source_bqm.vartype
 
-        # create a new empty binary quadratic model with the same class as
-        # source_bqm if it's shapeable, otherwise use AdjVectorBQM
-        if source_bqm.shapeable():
-            target_bqm = source_bqm.base.empty(smear_vartype)
-        else:
+        # we need this check to support dimod 0.9.x, where there was one bqm
+        # type that was not shapeable
+        if (isinstance(source_bqm, (AdjArrayBQM, SpinView, BinaryView)) and
+                not source_bqm.shapeable()):
+            # this will never happen in dimod>=0.10.0
             target_bqm = dimod.AdjVectorBQM.empty(smear_vartype)
+        else:
+            try:
+                # dimod 0.9.x
+                target_bqm = source_bqm.base.empty(smear_vartype)
+            except AttributeError:
+                target_bqm = source_bqm.empty(smear_vartype)
 
         # add the offset
-        target_bqm.add_offset(source_bqm.offset)
+        target_bqm.offset += source_bqm.offset
 
         if chain_strength is None:
             chain_strength = uniform_torque_compensation
@@ -275,7 +303,7 @@ class EmbeddedStructure(dict):
                     target_bqm.add_variable(p, 2*strength)
                     target_bqm.add_variable(q, 2*strength)
 
-        target_bqm.add_offset(offset)
+        target_bqm.offset += offset
 
         # next up the quadratic biases, spread the quadratic biases evenly over
         # the available interactions
@@ -316,12 +344,10 @@ def embed_bqm(source_bqm, embedding=None, target_adjacency=None,
             EmbeddedStructure object.
 
         chain_strength (float/mapping/callable, optional):
-            Magnitude of the quadratic bias (in SPIN-space) applied between
-            variables to form a chain, with the energy penalty of chain breaks
-            set to 2 * `chain_strength`.  If a mapping is passed, a chain-specific
-            strength is applied.  If a callable is passed, it will be called on
-            `chain_strength(source_bqm, embedding)` and should return a
-            float or mapping, to be interpreted as above. By default,
+            Sets the coupling strength between qubits representing variables 
+            that form a :term:`chain`. Mappings should specify the required chain
+            strength for each variable. Callables should accept the BQM and 
+            embedding and return a float or mapping. By default, 
             `chain_strength` is calculated with
             :func:`~dwave.embedding.chain_strength.uniform_torque_compensation`.
 
@@ -366,7 +392,7 @@ def embed_bqm(source_bqm, embedding=None, target_adjacency=None,
                 "target_adjacency should not be provided if embedding is an "
                 "EmbeddedStructure. The given value will be ignored. In the "
                 "future this will raise an exception",
-                DeprecationWarning
+                DeprecationWarning, stacklevel=2
                 )
     elif target_adjacency is None:
         raise ValueError("either embedding should be an EmbeddedStructure, or "
@@ -399,13 +425,11 @@ def embed_ising(source_h, source_J, embedding, target_adjacency, chain_strength=
             where t is a target-graph variable and Nt is its set of neighbours.
 
         chain_strength (float/mapping/callable, optional):
-            Magnitude of the quadratic bias (in SPIN-space) applied between
-            variables to form a chain, with the energy penalty of chain breaks
-            set to 2 * `chain_strength`.  If a mapping is passed, a chain-specific
-            strength is applied.  If a callable is passed, it will be called on
-            `chain_strength(source_bqm, embedding)` and should return a
-            float or mapping, to be interpreted as above. By default,
-            `chain_strength` is calculated with 
+            Sets the coupling strength between qubits representing variables 
+            that form a :term:`chain`. Mappings should specify the required chain
+            strength for each variable. Callables should accept the BQM and 
+            embedding and return a float or mapping. By default, 
+            `chain_strength` is calculated with
             :func:`~dwave.embedding.chain_strength.uniform_torque_compensation`.
 
     Returns:
@@ -462,13 +486,11 @@ def embed_qubo(source_Q, embedding, target_adjacency, chain_strength=None):
             where t is a target-graph variable and Nt is its set of neighbours.
 
         chain_strength (float/mapping/callable, optional):
-            Magnitude of the quadratic bias (in SPIN-space) applied between
-            variables to form a chain, with the energy penalty of chain breaks
-            set to 2 * `chain_strength`.  If a mapping is passed, a
-            chain-specific strength is applied.  If a callable is passed, it
-            will be called on `chain_strength(source_bqm, embedding)`
-            and should return a float or mapping, to be interpreted as above.
-            By default, `chain_strength` is calculated with 
+            Sets the coupling strength between qubits representing variables 
+            that form a :term:`chain`. Mappings should specify the required chain
+            strength for each variable. Callables should accept the BQM and 
+            embedding and return a float or mapping. By default, 
+            `chain_strength` is calculated with
             :func:`~dwave.embedding.chain_strength.uniform_torque_compensation`.
 
     Returns:
@@ -581,7 +603,7 @@ def unembed_sampleset(target_sampleset, embedding, source_bqm,
 
         # Add a new data field tracking which came from
         # todo: add this functionality to dimod
-        cbm_idxs = np.empty(len(sampleset), dtype=np.int)
+        cbm_idxs = np.empty(len(sampleset), dtype=int)
 
         start = 0
         for i, ss in enumerate(samplesets):
