@@ -12,8 +12,7 @@
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
-#
-# =============================================================================
+
 """Embedding composites for various types of problems and application.
 For example:
 
@@ -24,6 +23,7 @@ For example:
 * :class:`.LazyFixedEmbeddingComposite` can benefit applications that
   resubmit a BQM with changes in some values.
 """
+
 import itertools
 
 from warnings import warn
@@ -143,7 +143,7 @@ class EmbeddingComposite(dimod.ComposedSampler):
     """
 
     warnings_default = WarningAction.IGNORE
-    """Defines the default behabior for :meth:`.sample`'s `warnings` kwarg.
+    """Defines the default behavior for :meth:`.sample`'s `warnings` kwarg.
     """
 
     def sample(self, bqm, chain_strength=None,
@@ -160,13 +160,11 @@ class EmbeddingComposite(dimod.ComposedSampler):
                 Binary quadratic model to be sampled from.
 
             chain_strength (float/mapping/callable, optional):
-                Magnitude of the quadratic bias (in SPIN-space) applied between
-                variables to create chains. The energy penalty of chain breaks
-                is 2 * `chain_strength`.  If a mapping is passed, a
-                chain-specific strength is applied.  If a callable is passed, it
-                will be called on `chain_strength(bqm, embedding)` and should
-                return a float or mapping, to be interpreted as above. By default,
-                `chain_strength` is calculated with 
+                Sets the coupling strength between qubits representing variables 
+                that form a :term:`chain`. Mappings should specify the required 
+                chain strength for each variable. Callables should accept the BQM 
+                and embedding and return a float or mapping. By default, 
+                `chain_strength` is calculated with
                 :func:`~dwave.embedding.chain_strength.uniform_torque_compensation`.
 
             chain_break_method (function/list, optional):
@@ -448,12 +446,10 @@ class LazyFixedEmbeddingComposite(EmbeddingComposite, dimod.Structured):
                 Binary quadratic model to be sampled from.
 
             chain_strength (float/mapping/callable, optional):
-                Magnitude of the quadratic bias (in SPIN-space) applied between
-                variables to form a chain, with the energy penalty of chain
-                breaks set to 2 * `chain_strength`.  If a mapping is passed, a
-                chain-specific strength is applied.  If a callable is passed, it
-                will be called on `chain_strength(bqm, embedding)` and should
-                return a float or mapping, to be interpreted as above. By default, 
+                Sets the coupling strength between qubits representing variables 
+                that form a :term:`chain`. Mappings should specify the required 
+                chain strength for each variable. Callables should accept the BQM 
+                and embedding and return a float or mapping. By default, 
                 `chain_strength` is calculated with
                 :func:`~dwave.embedding.chain_strength.uniform_torque_compensation`.
 
@@ -549,13 +545,12 @@ class FixedEmbeddingComposite(LazyFixedEmbeddingComposite):
         # dev note: this entire block is to support a deprecated feature and can
         # be removed in the next major release
         if embedding is None:
-
-            warn(("The source_adjacency parameter is deprecated"),
-                 DeprecationWarning)
-
             if source_adjacency is None:
                 raise TypeError("either embedding or source_adjacency must be "
                                 "provided")
+            else:
+                warn(("The source_adjacency parameter is deprecated"),
+                     DeprecationWarning, stacklevel=2)
 
             source_edgelist = [(u, v) for u in source_adjacency for v in source_adjacency[u]]
 
