@@ -11,13 +11,12 @@
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
-#
-# =============================================================================
+
 __all__ = ['__version__', '__author__', '__authoremail__', '__description__']
 
-__version__ = '1.2.1'
+__version__ = '1.16.0'
 __author__ = 'D-Wave Systems Inc.'
-__authoremail__ = 'acondello@dwavesys.com'
+__authoremail__ = 'tools@dwavesys.com'
 __description__ = 'All things D-Wave System.'
 
 
