@@ -11,102 +11,118 @@
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
-#
-# ================================================================================================
+
 """
 A :std:doc:`dimod sampler <oceandocs:docs_dimod/reference/samplers>` for the D-Wave system.
 
 See :std:doc:`Ocean Glossary <oceandocs:glossary>`
 for explanations of technical terms in descriptions of Ocean tools.
 """
-from __future__ import division
 
-import time
-import functools
 import collections.abc as abc
 
-from warnings import warn
-
 import dimod
 
 from dimod.exceptions import BinaryQuadraticModelStructureError
-from dwave.cloud import Client
-from dwave.cloud.exceptions import SolverOfflineError, SolverNotFoundError
+from dwave.cloud.client import Client
+from dwave.cloud.exceptions import (
+    SolverError, SolverAuthenticationError, InvalidAPIResponseError,
+    RequestTimeout, PollingTimeout, ProblemUploadError, ProblemStructureError,
+)
 
+from dwave.system.exceptions import FailoverCondition, RetryCondition
 from dwave.system.warnings import WarningHandler, WarningAction
 
 import dwave_networkx as dnx
+import networkx as nx
 
-__all__ = ['DWaveSampler']
+__all__ = ['DWaveSampler', 'qpu_graph']
 
 
-def _failover(f):
-    @functools.wraps(f)
-    def wrapper(sampler, *args, **kwargs):
-        while True:
-            try:
-                return f(sampler, *args, **kwargs)
-            except SolverOfflineError as err:
-                if not sampler.failover:
-                    raise err
+def qpu_graph(topology_type, topology_shape, nodelist, edgelist):
+    """Converts node and edge lists to a dwave-networkx compatible graph.
 
-            try:
-                # the requested features are saved on the client object, so
-                # we just need to request a new solver
-                sampler.solver = sampler.client.get_solver()
+    Creates a D-Wave Chimera, Pegasus or Zephyr graph compatible with
+    dwave-networkx libraries. 
 
-                # delete the lazily-constructed attributes
-                try:
-                    del sampler._edgelist
-                except AttributeError:
-                    pass
-
-                try:
-                    del sampler._nodelist
-                except AttributeError:
-                    pass
-
-                try:
-                    del sampler._parameters
-                except AttributeError:
-                    pass
-
-                try:
-                    del sampler._properties
-                except AttributeError:
-                    pass
-
-            except SolverNotFoundError as err:
-                if sampler.retry_interval < 0:
-                    raise err
-
-                time.sleep(sampler.retry_interval)
-    return wrapper
+    Args:
+        topology_type (string):
+            The type of lattice. Valid strings are `chimera`, `pegasus`
+            and `zephyr`.
+        topology_shape(iterable of ints):
+            Specifies dimensions of the lattice.
+        nodelist (list of ints):
+            List of nodes in the graph. Node labeling is integer,
+            and compatible with the topology_type linear labeling scheme.
+        edgelist (list of Tuples):
+            List of edges in the graph, each edge consisting of a pair
+            of nodes.
+    """
+    
+    if topology_type == 'chimera':
+        if not (1 <= len(topology_shape) <=3):
+            raise ValueError('topology_shape is incompatible with a chimera lattice.')
+        G = dnx.chimera_graph(*topology_shape,
+                              node_list=nodelist,
+                              edge_list=edgelist)
+    elif topology_type == 'pegasus':
+        if len(topology_shape) != 1:
+            raise ValueError('topology_shape is incompatible with a pegasus lattice.')
+        G = dnx.pegasus_graph(topology_shape[0],
+                                  node_list=nodelist,
+                                  edge_list=edgelist)
+    elif topology_type == 'zephyr':
+        if len(topology_shape) not in (1, 2):
+            raise ValueError('topology_shape is incompatible with a zephyr lattice.')
+        G = dnx.zephyr_graph(*topology_shape,
+                                 node_list=nodelist,
+                                 edge_list=edgelist)
+    else:
+        # Alternative could be to create a standard network graph and
+        # issue a warning. Requires new dependency on networkx.
+        raise ValueError('topology_type does not match a known QPU architecure')
+    return G
 
 
 class DWaveSampler(dimod.Sampler, dimod.Structured):
     """A class for using the D-Wave system as a sampler for binary quadratic models.
 
-    Uses parameters set in a configuration file, as environment variables, or
-    explicitly as input arguments for selecting and communicating with a D-Wave
-    system. For more information, see
-    `D-Wave Cloud Client <https://docs.ocean.dwavesys.com/en/stable/docs_cloud/sdk_index.html>`_.
+    You can configure your :term:`solver` selection and usage by setting parameters,
+    hierarchically, in a configuration file, as environment variables, or
+    explicitly as input arguments. For more information, see
+    `D-Wave Cloud Client <https://docs.ocean.dwavesys.com/en/stable/docs_cloud/sdk_index.html>`_
+    :meth:`~dwave.cloud.client.Client.get_solvers`. By default, online
+    D-Wave systems are returned ordered by highest number of qubits.
 
     Inherits from :class:`dimod.Sampler` and :class:`dimod.Structured`.
 
     Args:
         failover (bool, optional, default=False):
-            Switch to a new QPU in the rare event that the currently connected
-            system goes offline. Note that different QPUs may have different
-            hardware graphs and a failover will result in a regenerated
-            :attr:`.nodelist`, :attr:`.edgelist`, :attr:`.properties` and
-            :attr:`.parameters`.
+            Signal a failover condition if a sampling error occurs. When ``True``,
+            raises :exc:`~dwave.system.exceptions.FailoverCondition` or
+            :exc:`~dwave.system.exceptions.RetryCondition` on sampleset resolve
+            to signal failover.
+
+            Actual failover, i.e. selection of a new solver, has to be handled
+            by the user. A convenience method :meth:`.trigger_failover` is available
+            for this. Note that hardware graphs vary between QPUs, so triggering
+            failover results in regenerated :attr:`.nodelist`, :attr:`.edgelist`,
+            :attr:`.properties` and :attr:`.parameters`.
+
+            .. versionchanged:: 1.16.0
+
+               In the past, the :meth:`.sample` method was blocking and
+               ``failover=True`` caused a solver failover and sampling retry.
+               However, this failover implementation broke when :meth:`sample`
+               became non-blocking (asynchronous), Setting ``failover=True`` had
+               no effect.
 
         retry_interval (number, optional, default=-1):
-            The amount of time (in seconds) to wait to poll for a solver in
-            the case that no solver is found. If `retry_interval` is negative
-            then it will instead propogate the `SolverNotFoundError` to the
-            user.
+            Ignored, but kept for backward compatibility.
+
+            .. versionchanged:: 1.16.0
+
+               Ignored since 1.16.0. See note for ``failover`` parameter above.
 
         **config:
             Keyword arguments passed to :meth:`dwave.cloud.client.Client.from_config`.
@@ -122,7 +138,7 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
         adjacent qubits on a D-Wave system. ``qubit_a`` is the first qubit in
         the QPU's indexed list of qubits and ``qubit_b`` is one of the qubits
         coupled to it. Other required parameters for communication with the system, such
-        as its URL and an autentication token, are implicitly set in a configuration file
+        as its URL and an authentication token, are implicitly set in a configuration file
         or as environment variables, as described in
         `Configuring Access to D-Wave Solvers <https://docs.ocean.dwavesys.com/en/stable/overview/sapi.html>`_.
         Given sufficient reads (here 100), the quantum
@@ -147,7 +163,6 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
 
     """
     def __init__(self, failover=False, retry_interval=-1, **config):
-
         # strongly prefer QPU solvers; requires kwarg-level override
         config.setdefault('client', 'qpu')
 
@@ -231,6 +246,7 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
             parameters = {param: ['parameters']
                           for param in self.properties['parameters']}
             parameters.update(warnings=[])
+            parameters.update(label=[])
             self._parameters = parameters
             return parameters
 
@@ -281,7 +297,34 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
             self._nodelist = nodelist = sorted(self.solver.nodes)
         return nodelist
 
-    @_failover
+    def trigger_failover(self):
+        """Trigger a failover and connect to a new solver."""
+
+        # the requested features are saved on the client object, so
+        # we just need to request a new solver
+        self.solver = self.client.get_solver(refresh=True)
+
+        # delete the lazily-constructed attributes
+        try:
+            del self._edgelist
+        except AttributeError:
+            pass
+
+        try:
+            del self._nodelist
+        except AttributeError:
+            pass
+
+        try:
+            del self._parameters
+        except AttributeError:
+            pass
+
+        try:
+            del self._properties
+        except AttributeError:
+            pass
+
     def sample(self, bqm, warnings=None, **kwargs):
         """Sample from the specified binary quadratic model.
 
@@ -302,10 +345,11 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
                 describes the parameters and properties supported on the D-Wave system.
 
         Returns:
-            :class:`dimod.SampleSet`: A `dimod` :obj:`~dimod.SampleSet` object.
+            :class:`~dimod.SampleSet`: Sample set constructed from a (non-blocking)
+            :class:`~concurrent.futures.Future`-like object.
             In it this sampler also provides timing information in the `info`
             field as described in the D-Wave System Documentation's
-            `timing guide <https://docs.dwavesys.com/docs/latest/doc_timing.html>`_.
+            :ref:`sysdocs_gettingstarted:qpu_sapi_qpu_timing`.
 
         Examples:
             This example submits a two-variable Ising problem mapped directly to two
@@ -335,29 +379,56 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
 
         solver = self.solver
 
-        if not (solver.nodes.issuperset(bqm.linear) and
-                solver.edges.issuperset(bqm.quadratic)):
-            msg = "Problem graph incompatible with solver."
-            raise BinaryQuadraticModelStructureError(msg)
-
-        future = solver.sample_bqm(bqm, **kwargs)
+        try:
+            future = solver.sample_bqm(bqm, **kwargs)
+        except ProblemStructureError as exc:
+            msg = ("Problem graph incompatible with solver. Please use 'EmbeddingComposite' "
+                   "to map the problem graph to the solver.")
+            raise BinaryQuadraticModelStructureError(msg) from exc
 
         if warnings is None:
             warnings = self.warnings_default
         warninghandler = WarningHandler(warnings)
         warninghandler.energy_scale(bqm)
 
-        # need a hook so that we can check the sampleset (lazily) for
-        # warnings
+        # need a hook so that we can lazily check the sampleset for warnings
+        # and handle failover consistently
         def _hook(computation):
-            sampleset = computation.sampleset
+            def resolve(computation):
+                sampleset = computation.sampleset
+                sampleset.resolve()
 
-            if warninghandler is not None:
-                warninghandler.too_few_samples(sampleset)
-                if warninghandler.action is WarningAction.SAVE:
-                    sampleset.info['warnings'] = warninghandler.saved
+                if warninghandler is not None:
+                    warninghandler.too_few_samples(sampleset)
+                    if warninghandler.action is WarningAction.SAVE:
+                        sampleset.info['warnings'] = warninghandler.saved
 
-            return sampleset
+                return sampleset
+
+            try:
+                return resolve(computation)
+
+            except (ProblemUploadError, RequestTimeout, PollingTimeout) as exc:
+                if not self.failover:
+                    raise exc
+
+                # failover with retry on:
+                # - request or polling timeout
+                # - upload errors
+                raise RetryCondition("resubmit problem") from exc
+
+            except (SolverError, InvalidAPIResponseError) as exc:
+                if not self.failover:
+                    raise exc
+                if isinstance(exc, SolverAuthenticationError):
+                    raise exc
+
+                # failover on:
+                # - solver offline, solver disabled or not found
+                # - internal SAPI errors (like malformed response)
+                # - generic solver errors
+                # but NOT on auth errors
+                raise FailoverCondition("switch solver and resubmit problem") from exc
 
         return dimod.SampleSet.from_future(future, _hook)
 
@@ -366,7 +437,8 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
         # on missing nodes for lists
         if isinstance(h, list):
             if len(h) > self.solver.num_qubits:
-                msg = "Problem graph incompatible with solver."
+                msg = ("Problem graph incompatible with solver. Please use 'EmbeddingComposite' "
+                       "to map the problem graph to the solver.")
                 raise BinaryQuadraticModelStructureError(msg)
             nodes = self.solver.nodes
             h = dict((v, b) for v, b in enumerate(h) if b and v in nodes)
@@ -470,13 +542,16 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
             if round(abs((s0 - s1) / (t0 - t1)),10) > max_slope:
                 raise ValueError("the maximum slope cannot exceed {}".format(max_slope))
 
-
+    
+        
+    
     def to_networkx_graph(self):
-        """Converts DWaveSampler's structure to a Chimera or Pegasus NetworkX graph.
+        """Converts DWaveSampler's structure to a Chimera, Pegasus or Zephyr NetworkX graph.
 
         Returns:
             :class:`networkx.Graph`:
-                Either an (m, n, t) Chimera lattice or a Pegasus lattice of size m.
+                Either a Chimera lattice of shape [m, n, t], a Pegasus 
+                lattice of shape [m] or a Zephyr lattice of size [m,t].
 
         Examples:
             This example converts a selected D-Wave system solver to a graph
@@ -489,18 +564,7 @@ class DWaveSampler(dimod.Sampler, dimod.Structured):
             >>> len(g.nodes) > 2000                  # doctest: +SKIP
             True
         """
-
-        topology_type = self.properties['topology']['type']
-        shape = self.properties['topology']['shape']
-
-        if topology_type == 'chimera':
-            G = dnx.chimera_graph(*shape,
-                                  node_list=self.nodelist,
-                                  edge_list=self.edgelist)
-
-        elif topology_type == 'pegasus':
-            G = dnx.pegasus_graph(shape[0],
-                                  node_list=self.nodelist,
-                                  edge_list=self.edgelist)
-
-        return G
+        return qpu_graph(self.properties['topology']['type'],
+                         self.properties['topology']['shape'],
+                         self.nodelist, self.edgelist)
+        
